from tottle.api.abc import ABCAPI
from tottle.dispatch.dispenser.abc import ABCStateDispenser
from tottle.dispatch.abc import ABCRouter
from tottle.dispatch.views import ABCView
from tottle.exception_factory import ABCErrorHandler
from tottle.modules import logger

from typing import Dict


class BotRouter(ABCRouter):
    async def route(self, update: dict, ctx_api: "ABCAPI"):
        logger.debug("Routing update {}".format(update))

        for view in self.views.values():
            try:
                if not await view.process_update(update):
                    continue
                await view.handle_update(update, ctx_api, self.state_dispenser)
            except BaseException as e:
                await self.error_handler.handle(e)

    def construct(
        self,
        views: Dict[str, "ABCView"],
        state_dispenser: ABCStateDispenser,
        error_handler: ABCErrorHandler,
    ) -> "BotRouter":
        self.views = views
        self.state_dispenser = state_dispenser
        self.error_handler = error_handler
        return self
