from typing import Union, Any, Callable

from .abc import ABCHandler
from ..rules import ABCRule


class FromFuncHandler(ABCHandler):
    def __init__(self, handler: Callable, *rules: "ABCRule", blocking: bool = True):
        self.handler = handler
        self.rules = rules
        self.blocking = blocking

    async def filter(self, update: Any) -> Union[dict, bool]:
        rule_context = {}
        for rule in self.rules:
            result = await rule.check(update)
            if result is False or result is None:
                return False
            elif result is True:
                continue
            rule_context.update(result)
        return rule_context

    async def handle(self, update: Any, **context) -> Any:
        return await self.handler(update, **context)

    def __repr__(self):
        return f"<FromFuncHandler {self.handler.__name__} blocking={self.blocking} rules={self.rules}>"
