from typing import List, Optional
from pydantic import Field, BaseModel

from tottle.types.objects.alert import ProximityAlertTriggered
from tottle.types.objects.animation import Animation
from tottle.types.objects.audio import Audio
from tottle.types.objects.chat import Chat
from tottle.types.objects.contact import Contact
from tottle.types.objects.dice import Dice
from tottle.types.objects.entity import Entity
from tottle.types.objects.game import Game
from tottle.types.objects.invoice import Invoice
from tottle.types.objects.keyboard import InlineKeyboardMarkup
from tottle.types.objects.location import Location
from tottle.types.objects.passport import PassportData
from tottle.types.objects.payment import SuccessfulPayment
from tottle.types.objects.photo import PhotoSize
from tottle.types.objects.poll import Poll
from tottle.types.objects.sticker import Sticker
from tottle.types.objects.user import User
from tottle.types.objects.venue import Venue
from tottle.types.objects.video import Video, VideoNote


class Message(BaseModel):
    chat: Optional["Chat"] = None
    date: Optional[int] = None
    message_id: Optional[int] = None
    from_user: Optional["User"] = Field(alias="from")

    forward_from: Optional["User"] = None
    forward_from_chat: Optional["Chat"] = None
    forward_from_message_id: Optional[int] = None
    forward_signature: Optional[str] = None
    forward_sender_name: Optional[str] = None
    forward_date: Optional[int] = None

    reply_to_message: Optional["Message"] = None
    via_bot: Optional["User"] = None
    edit_date: Optional[int] = None

    media_group_id: Optional[str] = None
    author_signature: Optional[str] = None
    text: Optional[str] = None
    entities: Optional[List["Entity"]] = None
    animation: Optional["Animation"] = None

    audio: Optional["Audio"] = None
    photo: Optional[List["PhotoSize"]] = None
    sticker: Optional["Sticker"] = None
    video: Optional["Video"] = None
    video_note: Optional["VideoNote"] = None

    caption: Optional[str] = None
    caption_entities: Optional[List["Entity"]] = None

    contact: Optional["Contact"] = None
    dice: Optional["Dice"] = None
    game: Optional["Game"] = None
    poll: Optional["Poll"] = None
    venue: Optional["Venue"] = None
    location: Optional["Location"] = None

    new_chat_members: Optional[List["User"]] = None
    new_chat_title: Optional[str] = None
    new_chat_photo: Optional[List["PhotoSize"]] = None
    delete_chat_photo: Optional[bool] = None
    group_chat_created: Optional[bool] = None
    supergroup_chat_created: Optional[bool] = None
    channel_chat_created: Optional[bool] = None

    migrate_to_chat_id: Optional[int] = None
    migrate_from_chat_id: Optional[int] = None

    pinned_message: Optional["Message"] = None
    invoice: Optional["Invoice"] = None
    successful_payment: Optional["SuccessfulPayment"] = None

    connected_website: Optional[str] = None

    passport_data: Optional["PassportData"] = None
    proximity_alert_triggered: Optional["ProximityAlertTriggered"] = None
    reply_markup: Optional["InlineKeyboardMarkup"] = None


Message.update_forward_refs()
