from typing import Optional

from pydantic import BaseModel, Field

from tottle.types.objects.message import Message
from tottle.types.objects.location import Location
from tottle.types.objects.order import ShippingAddress, OrderInfo
from tottle.types.objects.user import User


class CallbackQuery(BaseModel):
    id: Optional[str] = None
    from_user: Optional[User] = Field(alias="from")
    message: Optional["Message"] = None
    inline_message_id: Optional[str] = None

    chat_instance: Optional[str] = None
    data: Optional[str] = None
    game_short_name: Optional[str] = None


class InlineQuery(BaseModel):
    id: Optional[str] = None
    from_user: Optional[User] = Field(alias="from")
    location: Optional[Location] = None

    query: Optional[str] = None
    offset: Optional[str] = None


class ShippingQuery(BaseModel):
    id: Optional[str] = None
    from_user: Optional[User] = Field(alias="from")
    invoice_payload: Optional[str] = None
    shipping_address: Optional[ShippingAddress] = None


class PreCheckoutQuery(BaseModel):
    id: Optional[str] = None
    from_user: Optional[User] = Field(alias="from")

    currency: Optional[str] = None
    total_amount: Optional[str] = None

    invoice_payload: Optional[str] = None
    shipping_option_id: Optional[str] = None
    order_info: Optional[OrderInfo] = None


class ChosenInlineResult(BaseModel):
    result_id: Optional[str] = None
    from_user: Optional[User] = Field(alias="from")
    location: Optional[Location] = None

    inline_message_id: Optional[str] = None
    query: Optional[str] = None
